# This file has been generated


build_system = 'AppVeyor'
build_id = '26241728'
build_number = '339'
build_version = '1.0.339'
job_id = '8ivvlj5eqd3hwtta'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/8ivvlj5eqd3hwtta'
